/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SimpleFinderSupport.java,v 3456677c87a6 2010/06/07 21:15:21 fritz $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.simple;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.util.Set;
import it.tidalwave.text.DisplayableComparator;
import it.tidalwave.util.As;
import static it.tidalwave.util.Displayable.Displayable;
import it.tidalwave.bluebill.observation.Finder;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import java.util.Collection;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class SimpleFinderSupport<T extends As> extends it.tidalwave.util.FinderSupport<T> implements Finder<T>
  {
    private static final DisplayableComparator DISPLAYABLE_COMPARATOR = DisplayableComparator.getInstance();

    private final static Map<Finder.SortCriterion, Comparator<?>> COMPARATOR_MAP_BY_CRITERION =
                 new HashMap<Finder.SortCriterion, Comparator<?>>();

    private static final Comparator<?> AS_DISPLAYABLE_COMPARATOR = new Comparator<As>()
      {
        public int compare (final @Nonnull As o1, final @Nonnull As o2)
          {
            return DISPLAYABLE_COMPARATOR.compare(o1.as(Displayable), o2.as(Displayable));
          }
      };

    private static final Comparator<?> OBSERVATION_DATE_COMPARATOR = new Comparator<Observation>()
      {
        public int compare (final @Nonnull Observation o1, final @Nonnull Observation o2)
          {
            return o1.getDate().compareTo(o2.getDate());
          }
      };

    private static final Comparator<?> OBSERVABLE_COMPARATOR = new Comparator<ObservationItem>()
      {
        public int compare (final @Nonnull ObservationItem o1, final @Nonnull ObservationItem o2)
          {
            return DISPLAYABLE_COMPARATOR.compare(o1.getObservable().as(Displayable),
                                                  o2.getObservable().as(Displayable));
          }
      };

    static
      {
        COMPARATOR_MAP_BY_CRITERION.put(Finder.SortCriterion.DISPLAY_NAME, AS_DISPLAYABLE_COMPARATOR);
        COMPARATOR_MAP_BY_CRITERION.put(Finder.SortCriterion.DATE, OBSERVATION_DATE_COMPARATOR);
        COMPARATOR_MAP_BY_CRITERION.put(Finder.SortCriterion.OBSERVABLE, OBSERVABLE_COMPARATOR);
      }

    @Nonnull
    private final List<T> contents = new ArrayList<T>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
//    public SimpleFinderSupport (final @Nonnull Set<T> contents)
//      {
//        this(new ArrayList<T>(contents));
//      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public SimpleFinderSupport (final @Nonnull Collection<T> contents)
      {
        super("xxx");
        this.contents.addAll(contents);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public Finder<T> clone()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder<T> from (final @Nonnegative int firstResult)
      {
        return (Finder<T>)super.from(firstResult);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder<T> max (final @Nonnegative int maxResults)
      {
        return (Finder<T>)super.max(maxResults);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Finder<T> sort (final @Nonnull Finder.SortCriterion criterion, final @Nonnull Finder.SortDirection direction)
      {
        final Comparator comparator = COMPARATOR_MAP_BY_CRITERION.get(criterion);

        if (comparator == null)
          {
            throw new IllegalArgumentException("Cannot sort by " + criterion);
          }
        
        final int sgn = (direction == Finder.SortDirection.ASCENDING) ? +1 : -1;

        Collections.sort(contents, new Comparator<T>()
          {
            public int compare (final @Nonnull T o1, final @Nonnull T o2)
              {
                return sgn * comparator.compare(o1, o2);
              }
          });

        return this;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Finder<T> sort (@Nonnull Finder.SortCriterion criterion)
      {
        return sort(criterion, SortDirection.ASCENDING);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    protected List<T> doCompute()
      {
        return contents;
      }
  }
